<h3>Index</h3>
<p><a href="{{ pathto('genindex') }}">Thesaurus</a></p>

{%- if show_source and has_source and sourcename %}
  <h3>{{ _('This Page') }}</h3>
  <ul class="this-page-menu">
    <li><a href="{{ pathto('_sources/' + sourcename, true)|e }}"
           rel="nofollow">{{ _('Show Source') }}</a></li>
   {% if display_github %}
     <li><a href="https://github.com/{{ github_user }}/{{ github_repo }}/blob/{{ github_version }}{{ conf_py_path }}{{ pagename }}.rst">
       Show on GitHub</a></li>
     <li><a href="https://github.com/{{ github_user }}/{{ github_repo }}/edit/{{ github_version }}{{ conf_py_path }}{{ pagename }}.rst">
       Edit on GitHub</a></li>
   {% endif %}
  </ul>
{%- endif %}
